/*
*Filename: mygui.c
*Author: Stanislav Subrt
*Year: 2013
*/

#include "gui.h"
#include "main.h"



//////////////////////////////////////////////////////////////////////
////////// INTERNAL TYPEDEFS /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


typedef struct{
	char *BtnCaption;
	int BtnEventID;					//ID of event that should happen after click
} BtnStructTd;



typedef struct {
	char ScreenTitle[32];
	void (*RedrawWindow)();		//pointer to function to draw/redraw screen
	BtnStructTd Btn0;
	BtnStructTd Btn1;
	BtnStructTd Btn2;
} ScreenStructTd;



typedef struct {
	ScreenStructTd *ScreenListArray;
	int NumberOfScreensInList;
	int ActiveScreenIndex;
	int IsRefreshNeeded;
} DispStructTd;




//////////////////////////////////////////////////////////////////////
////////// INTERNAL DECLARATIONS /////////////////////////////////////
//////////////////////////////////////////////////////////////////////

static void mygui_DrawScreenButtons(char* screenBuffer, ScreenStructTd* screenStruct);
static void mygui_DrawTitleAndBtns(void);
static void mygui_DrawMeasurementProgress(void);

static void mygui_ShowNextScreen(void);
static void mygui_PlaceTextSimple(char *text, unsigned x, unsigned y, unsigned IsColorInverted);


static int mygui_GetBtnEventID(BtnStructTd *btnStruct);
static void mygui_CallEventByID(int eventID);



static void DCTestScreenRedraw(void);
static void DCCrackDetectionScreenRedraw(void);
static void DCCrackMeasurementScreenRedraw(void);



#define STRUCTCOUNT_IN_ARRAY(x)	( (sizeof(x)) / (sizeof(x[0])) )
#define DEFAULT_SCREEN_INDEX	0

#define REFRESH_NEEDED			1
#define REFRESH_NOT_NEEDED		0


///////////////////////////////////////////////////////////////////////////////////
////////// HERE DEFINE EACH SCREEN STRUCT AND ADD INTO DISPSTRUCT /////////////////
///////////////////////////////////////////////////////////////////////////////////

//image buffer for our display
char ScreenBufferArray[LCD_SCREENBUFFERSIZE];

//event IDs - every button pressed has specified EventID for active screen
#define GUI_EVENTID_NONE						-1
#define GUI_EVENTID_STOPALL					0
#define GUI_EVENTID_CHANGEMODE				1
#define GUI_EVENTID_STARTDCREF				2
#define GUI_EVENTID_STARTDCTEST				3
#define GUI_EVENTID_STARTDCCRACKDETECTION		4
#define GUI_EVENTID_STARTDCCRACKMEASUREMENT	6



//SCREENSTRUCTS - screentitle cannot be longer than 20 chars
#define DCTESTSCREENSTRUCT				{"DC TEST", 				DCTestScreenRedraw,						{"Start", GUI_EVENTID_STARTDCTEST}, 				{"", GUI_EVENTID_NONE}, 				{"Mode", GUI_EVENTID_CHANGEMODE} }
#define DCCRACKDETECTIONSTRUCT			{"DC CRACK DETECT",			DCCrackDetectionScreenRedraw,			{"Start", GUI_EVENTID_STARTDCCRACKDETECTION}, 	{"Ref", GUI_EVENTID_STARTDCREF}, 		{"Mode", GUI_EVENTID_CHANGEMODE} }
#define DCCRACKMEASUREMENTSTRUCT		{"DC CRACK MEASUREMENT",	DCCrackMeasurementScreenRedraw,			{"Start", GUI_EVENTID_STARTDCCRACKMEASUREMENT}, 	{"", GUI_EVENTID_NONE}, 				{"Mode", GUI_EVENTID_CHANGEMODE} }

//SCREENSTRUCT LIST
ScreenStructTd ScreenList[] = {DCTESTSCREENSTRUCT, DCCRACKDETECTIONSTRUCT, DCCRACKMEASUREMENTSTRUCT};	// 	//screen structs aded into array

//DISPSTRUCT
DispStructTd DispStruct = {ScreenList, STRUCTCOUNT_IN_ARRAY(ScreenList), DEFAULT_SCREEN_INDEX, REFRESH_NEEDED};


//////////////////////////////////////////////////////////////////////
////////// EXTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//initialize gui module, lcd and buttons
void gui_Init(void){
	lcd_Init();
	btns_Init();
}



//request lcd redraw
void gui_RequestRefresh(void){
	DispStruct.IsRefreshNeeded = 1;
}



//handles button events and display refresh requested event
void gui_HandleRefreshRequests(void){
	int PressedButton;
	int EventID;

	while( (PressedButton = btns_GetPressedID()) != BTNS_NONE ){			//read all pressed buttons from queue
		switch( PressedButton ){											//for each of pressed buttons select proper event ID
			case BTNS_ID_BTN0:
				EventID = mygui_GetBtnEventID( &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].Btn0) );
			break;

			case BTNS_ID_BTN1:
				EventID = mygui_GetBtnEventID( &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].Btn1) );
			break;

			case BTNS_ID_BTN2:
				EventID = mygui_GetBtnEventID( &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].Btn2) );
			break;
		}
		mygui_CallEventByID(EventID);										//call event by ID
	}

	if( DispStruct.IsRefreshNeeded ){

		DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].RedrawWindow();	//call active screens redraw function

		lcd_UpdateScreen(ScreenBufferArray);
		DispStruct.IsRefreshNeeded = 0;
	}
}



//////////////////////////////////////////////////////////////////////
////////// BUTTON CALLBACKS AND REDRAW FUNCTIONS /////////////////////
//////////////////////////////////////////////////////////////////////

static void DCTestScreenRedraw(void){
	char tmp[500];
	float Ch1P, Ch1M, Ch2P, Ch2M, Sub1, Sub2;
	pm_MeasResultsTd* pResult;

	mygui_DrawTitleAndBtns();
	mygui_DrawMeasurementProgress();

	//print column captions
	mygui_PlaceTextSimple("Pol+:", 0, 1, 0);
	mygui_PlaceTextSimple("Pol-:", 4, 1, 0);
	mygui_PlaceTextSimple("Sub:", 8, 1, 0);

	//get pointer to measurement results struct of currently running or ready measurement
	pResult = pm_GetCurrentMeasurementResults(pm_GetRunningCycle());	//get values from running measuremen
	if( pResult == 0 ) {

		pResult =  pm_GetCurrentMeasurementResults(pm_GetLatestCycle());
		if( pResult == 0 ){
			return;
		}
	}

	//read test results for both channels and polarities
	Ch2M = pResult->MeasResultsArray[3];
	Ch2P = pResult->MeasResultsArray[2];
	Ch1M = pResult->MeasResultsArray[1];
	Ch1P = pResult->MeasResultsArray[0];

	//draw test results from channel 1 pol+ pol- sub/2
	//draw test results from channel 2 pol+ pol- sub/2
	switch(pResult->ReadyIndex){
		case 4:
			Sub2 = (Ch2P-Ch2M)/2;

			sprintf(tmp, "%.1f", Ch2M);
			mygui_PlaceTextSimple(tmp, 4, 3, 0);

			sprintf(tmp, "%.1f", Sub2);
			mygui_PlaceTextSimple(tmp, 8, 3, 0);


			//evaluate test
			if( abs((int)Ch2M) < 1000 && abs((int)Ch2P) < 1000 && abs((int)Ch1M) < 1000 && abs((int)Ch1P) < 1000){
				sprintf(tmp, "Gain too low!");
			}
			else if( abs((int)Ch2M) > 3000 || abs((int)Ch2P) > 3000 || abs((int)Ch1M) > 3000 || abs((int)Ch1P) > 3000){
				sprintf(tmp, "Gain too high!");
			}
			else{
				sprintf(tmp, "Gain OK!");
			}
			mygui_PlaceTextSimple(tmp, 0, 4, 0);


			if( ((Ch1P + Ch1M)/2 + (Ch2P + Ch2M)/2) > 2000 ||  ( (Ch1P + Ch1M)/2 + (Ch2P + Ch2M)/2 ) < 2000 ){
				sprintf(tmp, "Tune zero!");
				mygui_PlaceTextSimple(tmp, 0, 5, 0);
			}


		case 3:
			sprintf(tmp, "%.1f", Ch2P);
			mygui_PlaceTextSimple(tmp, 0, 3, 0);

		case 2:
			Sub1 = (Ch1P-Ch1M)/2;

			sprintf(tmp, "%.1f", Ch1M);
			mygui_PlaceTextSimple(tmp, 4, 2, 0);

			sprintf(tmp, "%.1f", Sub1);
			mygui_PlaceTextSimple(tmp, 8, 2, 0);

		case 1:
			sprintf(tmp, "%.1f", Ch1P);
			mygui_PlaceTextSimple(tmp, 0, 2, 0);
	}

	gui_RequestRefresh();
}



static void DCCrackDetectionScreenRedraw(void){
	char tmp[500];

	mygui_DrawTitleAndBtns();
	mygui_DrawMeasurementProgress();

	//draw column headings
	mygui_PlaceTextSimple("Sub:", 0, 1, 0);
	mygui_PlaceTextSimple("Ref:", 4, 1, 0);
	mygui_PlaceTextSimple("Ratio:", 8, 1, 0);


	//draw reference values
	if(pm_CrackReferenceMeasurementData.Ch1.Sub.IsValid){
		sprintf(tmp, "%.1f", pm_CrackReferenceMeasurementData.Ch1.Sub.Value);
		mygui_PlaceTextSimple(tmp, 4, 2, 0);
	}

	if(pm_CrackReferenceMeasurementData.Ch2.Sub.IsValid){
		sprintf(tmp, "%.1f", pm_CrackReferenceMeasurementData.Ch2.Sub.Value);
		mygui_PlaceTextSimple(tmp, 4, 3, 0);
	}

	//draw measurement values
	if(pm_CrackDetectionData.Ch1.Sub.IsValid){
		sprintf(tmp, "%.1f", pm_CrackDetectionData.Ch1.Sub.Value);
		mygui_PlaceTextSimple(tmp, 0, 2, 0);
	}

	if(pm_CrackDetectionData.Ch2.Sub.IsValid){
		//draw reference values
		sprintf(tmp, "%.1f", pm_CrackDetectionData.Ch2.Sub.Value);
		mygui_PlaceTextSimple(tmp, 0, 3, 0);
	}

	//draw meas/ref ratio
	if(pm_CrackDetectionData.Ch1.Sub.IsValid && pm_CrackReferenceMeasurementData.Ch1.Sub.IsValid){
		sprintf(tmp, "%.3f", pm_CrackDetectionData.Ch1.Sub.Value / pm_CrackReferenceMeasurementData.Ch1.Sub.Value);
		mygui_PlaceTextSimple(tmp, 8, 2, 0);
	}

	if(pm_CrackDetectionData.Ch2.Sub.IsValid && pm_CrackReferenceMeasurementData.Ch2.Sub.IsValid){
		sprintf(tmp, "%.3f", pm_CrackDetectionData.Ch2.Sub.Value / pm_CrackReferenceMeasurementData.Ch2.Sub.Value);
		mygui_PlaceTextSimple(tmp, 8, 3, 0);
	}

	gui_RequestRefresh();
}



static void DCCrackMeasurementScreenRedraw(void){
	char tmp[500];
	float A, a;
	float citatel, jmenovatel;

	mygui_DrawTitleAndBtns();
	mygui_DrawMeasurementProgress();


	//draw column headings
	mygui_PlaceTextSimple("Pol+:", 0, 1, 0);
	mygui_PlaceTextSimple("Pol-:", 4, 1, 0);
	mygui_PlaceTextSimple("Sub:", 8, 1, 0);


	//draw channel 1 values
	if(pm_CrackMeasData.Ch1.PolP.IsValid){
		sprintf(tmp, "%.1f", pm_CrackMeasData.Ch1.PolP.Value);
		mygui_PlaceTextSimple(tmp, 0, 2, 0);
	}

	if(pm_CrackMeasData.Ch1.PolM.IsValid){
		sprintf(tmp, "%.1f", pm_CrackMeasData.Ch1.PolM.Value);
		mygui_PlaceTextSimple(tmp, 4, 2, 0);

		sprintf(tmp, "%.1f", pm_CrackMeasData.Ch1.Sub.Value);
		mygui_PlaceTextSimple(tmp, 8, 2, 0);
	}

	//draw channel 2 values
	if(pm_CrackMeasData.Ch2.PolP.IsValid){
		sprintf(tmp, "%.1f", pm_CrackMeasData.Ch2.PolP.Value);
		mygui_PlaceTextSimple(tmp, 0, 3, 0);
	}

	if(pm_CrackMeasData.Ch2.PolM.IsValid){
		sprintf(tmp, "%.1f", pm_CrackMeasData.Ch2.PolM.Value);
		mygui_PlaceTextSimple(tmp, 4, 3, 0);

		sprintf(tmp, "%.1f", pm_CrackMeasData.Ch2.Sub.Value);
		mygui_PlaceTextSimple(tmp, 8, 3, 0);
	}

	//place size constant A = Sub1/Sub2; and calculate crack length for given sensor
	if( pm_CrackMeasData.Ch1.Sub.IsValid && pm_CrackMeasData.Ch2.Sub.IsValid ){
			#define SENSOR_Y1	(float)2
			#define SENSOR_Y2	(float)6
		A = pm_CrackMeasData.Ch1.Sub.Value / pm_CrackMeasData.Ch2.Sub.Value;


		a = 0;		//formula removed in public version

		sprintf(tmp, "A=%.3f;[-]", A);
		mygui_PlaceTextSimple(tmp, 0, 4, 0);

		sprintf(tmp, "a=%.2f; Y1=2 Y2=6", a);
		mygui_PlaceTextSimple(tmp, 0, 5, 0);
	}
	gui_RequestRefresh();
}



//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#define GUI_LCD_XGRID		12
#define GUI_LCD_YGRID		8
//draw text to display buffer, coords are in 12x8 grid
//x... <0..11>
//y... <0..7>
static void mygui_PlaceTextSimple(char *text, unsigned x, unsigned y, unsigned IsColorInverted){
	lcd_PositionTd StartPos;
	if(x>GUI_LCD_XGRID-1) x=GUI_LCD_XGRID-1;
	if(y>GUI_LCD_YGRID-1) y=GUI_LCD_YGRID-1;

	StartPos.x = x * LCD_WIDTH / GUI_LCD_XGRID;
	StartPos.page = y;
	lcd_PutStr(ScreenBufferArray, text, strlen(text), &StartPos, IsColorInverted);
}



//show next screen in screenlist
static void mygui_ShowNextScreen(void){

	DispStruct.ActiveScreenIndex++;
	if( DispStruct.ActiveScreenIndex >= DispStruct.NumberOfScreensInList ){	//if index after increment points behind list
		DispStruct.ActiveScreenIndex = 0;
	}
	gui_RequestRefresh();
}



//draw title and btn captions on screen
static void mygui_DrawTitleAndBtns(void){
	lcd_PositionTd StartPos = {0, 0};
	lcd_PositionTd EndPos = {127, 0};
	int i;

	//clear screen
	lcd_Fill(ScreenBufferArray, 0);

	//place title bar
	lcd_PlaceBackground(ScreenBufferArray, &StartPos, &EndPos, 0xFF);
	i = strlen(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].ScreenTitle);
	if( i>20 ) i = 20;
	lcd_PutStr(ScreenBufferArray, DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].ScreenTitle, i, &StartPos, 1);	//add screen caption with inverted colors to fit placed background
	mygui_PlaceTextSimple(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex].ScreenTitle, 0, 0, 1);

	//draw buttons
	mygui_DrawScreenButtons(ScreenBufferArray, &(DispStruct.ScreenListArray[DispStruct.ActiveScreenIndex]));
}



//place measurement progress indicator
static void mygui_DrawMeasurementProgress(void){
	int CycleProgress;
	char tmp[50];

	CycleProgress = pm_GetRunningCycleProgress();			//percentage
	if(CycleProgress == POTMET_CYCLE_NOT_RUNNING){
		sprintf(tmp, "Aperture ready!");
	}
	else{
		sprintf(tmp, "Measuring!");
	}
	mygui_PlaceTextSimple(tmp, 0, 6, 0);
}



//draw buttons on screen
static void mygui_DrawScreenButtons(char* screenBuffer, ScreenStructTd* screenStruct){
	lcd_PositionTd BgStart = {0, 7};
	lcd_PositionTd BgEnd = {127, 7};


	lcd_PlaceBackground(screenBuffer, &BgStart, &BgEnd, 0xFF);
	mygui_PlaceTextSimple(screenStruct->Btn0.BtnCaption, 0, 7, 1);
	mygui_PlaceTextSimple(screenStruct->Btn1.BtnCaption, 4, 7, 1);
	mygui_PlaceTextSimple(screenStruct->Btn2.BtnCaption, 8, 7, 1);
}



//get button EventID
static int mygui_GetBtnEventID(BtnStructTd *btnStruct){
	return btnStruct->BtnEventID;
}



//call event specified by ID
static void mygui_CallEventByID(int eventID){
	switch (eventID){
		case GUI_EVENTID_STOPALL:
			pm_StopMeasCycle();

			gui_RequestRefresh();
		break;

		case GUI_EVENTID_CHANGEMODE:
			pm_StopMeasCycle();
			mygui_ShowNextScreen();
			pm_ClearLatestCycleData();

			btns_ClearQueue();
			gui_RequestRefresh();
		break;

		case GUI_EVENTID_STARTDCTEST:
			pm_StartMeasCycle(&pm_DCTestMeasCycle);

			gui_RequestRefresh();
		break;


		case GUI_EVENTID_STARTDCREF:
			pm_StartMeasCycle(&pm_DCRefMeasCycle);
			pm_ClearSingleMeasurementStruct(&pm_CrackReferenceMeasurementData);
			gui_RequestRefresh();
		break;

		case GUI_EVENTID_STARTDCCRACKDETECTION:
			pm_StartMeasCycle(&pm_DCCrackDetectMeasCycle);
			pm_ClearSingleMeasurementStruct(&pm_CrackDetectionData);
			gui_RequestRefresh();
		break;

		case GUI_EVENTID_STARTDCCRACKMEASUREMENT:
			pm_StartMeasCycle(&pm_DCCrackLengthMeasCycle);
			pm_ClearSingleMeasurementStruct(&pm_CrackMeasData);
			gui_RequestRefresh();
		break;
	}
}
