/*
*Filename: mylcd.c
*Author: Stanislav Subrt
*Year: 2013
*/


#include "lcd.h"
#include "main.h"
#include "font5x8.h"


//////////////////////////////////////////////////////////////////////
////////// FUNCTION PROTOTYPES ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////

inline void lcd_Strobe(void);
inline void lcd_SetDataPins(unsigned data);

#define LCD_SELECTCHIP_NONE	0
#define LCD_SELECTCHIP_CS1	1
#define LCD_SELECTCHIP_CS2	2
#define LCD_SELECTCHIP_BOTH	3
inline void lcd_SelectChip(int chipSelect);

#define LCD_SELECTCMD_DATA		1
#define LCD_SELECTCMD_INSTRUCTION	0
#define LCD_SELECTCMD_READ	1
#define LCD_SELECTCMD_WRITE	0
inline void lcd_SelectCmd(int DI, int RW);

#define LCD_READY		0
#define LCD_BUSY		1
int lcd_IsControllerBusy(void);

#define LCD_SETONOFF_ON	1
#define LCD_SETONOFF_OFF	0
void lcd_SetOnOff(unsigned status);
void lcd_SetX(unsigned x);
void lcd_SetPage(unsigned page);
void lcd_WriteData(unsigned data);



//////////////////////////////////////////////////////////////////////
////////// SOURCE BODY ///////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//initialize LCD peripheral - setup bus and enable
void lcd_Init(void){
	GPIO_InitTypeDef GPIOInitStruct;

	//initialize LCD pins as pushpull outputs
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

	GPIOInitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIOInitStruct.GPIO_OType = GPIO_OType_PP;
	GPIOInitStruct.GPIO_Pin = LCD_INTERFACEPINS;
	GPIOInitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIOInitStruct.GPIO_Speed = GPIO_Speed_Level_3;
	GPIO_Init(LCD_PORT, &GPIOInitStruct);

	//enable display
	lcd_SetOnOff(LCD_SETONOFF_ON);
}


void lcd_SetOnOff(unsigned status){
	unsigned tmp = (LCD_PIN_D5 | LCD_PIN_D4 | LCD_PIN_D3 | LCD_PIN_D2 | LCD_PIN_D1);

	if(status){
		tmp |= LCD_PIN_D0;	//add display_on flag
	}

	lcd_SelectChip(LCD_SELECTCHIP_BOTH);
	lcd_SelectCmd(LCD_SELECTCMD_INSTRUCTION, LCD_SELECTCMD_WRITE);
	lcd_SetDataPins(tmp);
	while(lcd_IsControllerBusy());
	lcd_Strobe();
}


void lcd_UpdateScreen(const char* screenBuffer){
	int x, page, i = 0;

	lcd_SetX(0);
	lcd_SetPage(0);

	//left part of LCD

	for( page=0; page<LCD_PAGES; page++ ){		//for each of 8 pages of display
		lcd_SetPage(page);
		for(x = 0; x<LCD_WIDTH; x++ ){			//for each of 128 columns
			if( x == 0) {
				lcd_SelectChip(LCD_SELECTCHIP_CS1);
			}
			else if( x==64 ){
				lcd_SelectChip(LCD_SELECTCHIP_CS2);
			}
			//mylcd_WriteData(i++);
			lcd_WriteData(screenBuffer[i++]);
		}
	}
}


//////////////////////////////////////////////////////////////////////
////////// FUNCTIONS WORKING WITH SCREEN BUFFER //////////////////////
//////////////////////////////////////////////////////////////////////


void lcd_Fill(char* screenBuffer, char pattern){
	int i;
	for( i=0; i<LCD_SCREENBUFFERSIZE; i++ ){
		screenBuffer[i] = pattern;
	}
}

void lcd_CopyBuffer(char* destBuffer, char* sourceBuffer){
	int i;

	for( i=0; i<LCD_SCREENBUFFERSIZE; i++ ){
		destBuffer[i] = sourceBuffer[i];
	}
}

#define MYLCD_CHARLENGTH	5
#define MYLCD_SPACEWIDTH	1
//returns pointer to CharPositionStructTypedef, where coords for next char in line should be placed
lcd_PositionTd* lcd_PutChar(char* screenBuffer, char charToPut, lcd_PositionTd* charPositionStruct, int colorsInverted){
	int i, ScreenBufferIndex;
	int IsNonPrintableCharacter = 0;
	int X, Page;
	static lcd_PositionTd PositionToReturn;
	int CharStorage;

	//handle invalid params
	if( screenBuffer == 0 ) return 0;
	if( charPositionStruct == 0 ) return 0;

	X = charPositionStruct->x;
	Page = charPositionStruct->page;

	//handle special or invalid chars
	if (charToPut == '\n'){
		Page++;
		IsNonPrintableCharacter = 1;
	}
	else if ( charToPut == '\r' ){
		X = 0;
		IsNonPrintableCharacter = 1;
	}
	else if( (charToPut > 0x80) || (charToPut < 0x20) ){
		charToPut = 0x80;	//error char - all black
	}

	//handle invalid coords
	if( (X + MYLCD_CHARLENGTH) > (LCD_WIDTH - 1) ){	//if char wont fit on selected page(line)
		X = 0;
		Page++;
		if( Page > (LCD_PAGES - 1) ){		//if page is out of range
			Page = 0;
		}
	}

	//skip printing if non printable character found
	if(!IsNonPrintableCharacter){
		ScreenBufferIndex = X + Page * LCD_WIDTH;

		for ( i=0; i<MYLCD_CHARLENGTH; i++ ){
			CharStorage = (font5x8[ ( charToPut - 0x20 ) * MYLCD_CHARLENGTH + i]);
			if(colorsInverted){
				CharStorage = ~CharStorage;
			}
			screenBuffer[ScreenBufferIndex++] = CharStorage;
		}
		for( i=0; i<MYLCD_SPACEWIDTH; i++){
			CharStorage = 0;
			if(colorsInverted){
				CharStorage = ~CharStorage;
			}
			screenBuffer[ScreenBufferIndex++] = CharStorage;	//add space
		}
		PositionToReturn.x = X + MYLCD_CHARLENGTH + MYLCD_SPACEWIDTH;
		PositionToReturn.page = Page;
	}
	else{
		PositionToReturn.x = X;
		PositionToReturn.page = Page;
	}


	return &PositionToReturn;
}


lcd_PositionTd* lcd_PutStr(char* screenBuffer, char* strToPut, int stringLength, lcd_PositionTd* firstCharPositionStruct, int colorsInverted){
	int i = 0;
	static lcd_PositionTd* PositionStruct;

	if( screenBuffer == 0 ) return 0;
	if( strToPut == 0 ) return 0;
	if( firstCharPositionStruct == 0 ) return 0;
	if( stringLength == 0 ) return firstCharPositionStruct;
	//put first char at param defined coords
	PositionStruct = lcd_PutChar(screenBuffer, strToPut[i++], firstCharPositionStruct, colorsInverted);

	if(PositionStruct == 0) return 0;

	for( i=i; i<stringLength; i++ ){
		PositionStruct = lcd_PutChar(screenBuffer, strToPut[i], PositionStruct, colorsInverted);
		if(PositionStruct == 0) return 0;
	}

	return PositionStruct;
}

//xStart, xEnd		<0;127>
//pageStart, pageEnd	<0;7>
void lcd_InvertPixels(char* screenBuffer, lcd_PositionTd* startPosStruct, lcd_PositionTd* endPosStruct){
	int StartIndex, EndIndex, tmp, i;

	StartIndex = startPosStruct->page * LCD_WIDTH + startPosStruct->x;
	EndIndex = endPosStruct->page * LCD_WIDTH + endPosStruct->x;

	//check if start address is lower then and switch it if needed
	if( StartIndex > EndIndex ) {
		tmp = StartIndex;
		StartIndex = EndIndex;
		EndIndex = tmp;
	}
	//else if (StartIndex == EndIndex) return;

	for( i = StartIndex; i<=EndIndex; i++ ){
		screenBuffer[i] = ~(screenBuffer[i]);
	}
}


void lcd_PlaceBackground(char* screenBuffer, lcd_PositionTd* startPosStruct, lcd_PositionTd* endPosStruct, int bgColor){
	int StartIndex, EndIndex, tmp, i;

	StartIndex = startPosStruct->page * LCD_WIDTH + startPosStruct->x;
	EndIndex = endPosStruct->page * LCD_WIDTH + endPosStruct->x;

	//check if start address is lower then and switch it if needed
	if( StartIndex > EndIndex ) {
		tmp = StartIndex;
		StartIndex = EndIndex;
		EndIndex = tmp;
	}
	//else if (StartIndex == EndIndex) return;

	bgColor &= 0xFF;	//mask out invalid colors

	for( i = StartIndex; i<=EndIndex; i++ ){
		screenBuffer[i] = bgColor;
	}
}


//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#define MYLCD_DELAY	1
void mylcd_wait(void){
	int i;
	for( i=0; i<MYLCD_DELAY; i++);
}

inline void lcd_Strobe(void){
	mylcd_wait();
	GPIO_SetBits(LCD_PORT, LCD_PIN_E);
	mylcd_wait();
	GPIO_ResetBits(LCD_PORT, LCD_PIN_E);
}


inline void lcd_SetDataPins(unsigned data){
	unsigned tmp;

	tmp = GPIO_ReadInputData(LCD_PORT) & (~LCD_DATAPINS);	//save port status and mask out data pins
	tmp |= (data & LCD_DATAPINS);		//make sure data param is not touching non data pins and write its value to tmp
	GPIO_Write(LCD_PORT, tmp);		//write tmp to LCD port
}


//#define MYLCD_SELECTCHIP_NONE	0
//#define MYLCD_SELECTCHIP_CS1	1
//#define MYLCD_SELECTCHIP_CS2	2
//#define MYLCD_SELECTCHIP_BOTH	3
//param chipSelect: 0 = both off
//					1 = CS1
//					2 = CS2
//					other = CS1 and CS2
inline void lcd_SelectChip(int chipSelect){
	if( chipSelect==1 ){
		GPIO_ResetBits(LCD_PORT, LCD_PIN_CS2);
		GPIO_SetBits(LCD_PORT, LCD_PIN_CS1);
	}
	else if(chipSelect==2){
		GPIO_ResetBits(LCD_PORT, LCD_PIN_CS1);
		GPIO_SetBits(LCD_PORT, LCD_PIN_CS2);
	}
	else if(chipSelect==0){
		GPIO_ResetBits(LCD_PORT, LCD_PIN_CS1 | LCD_PIN_CS2);
	}
	else{
		GPIO_SetBits(LCD_PORT, LCD_PIN_CS1 | LCD_PIN_CS2);
	}
}

inline void lcd_SelectCmd(int DI, int RW){
	if(DI){
		GPIO_SetBits(LCD_PORT, LCD_PIN_DI);
	}
	else{
		GPIO_ResetBits(LCD_PORT, LCD_PIN_DI);
	}

	if(RW){
		GPIO_SetBits(LCD_PORT, LCD_PIN_RW);
	}
	else{
		GPIO_ResetBits(LCD_PORT, LCD_PIN_RW);
	}
}


//#define MYLCD_READY		0
//#define MYLCD_BUSY		1
#define MYLCD_STATUSPINS 	( LCD_PIN_D7 | LCD_PIN_D5 | LCD_PIN_D4 )
int lcd_IsControllerBusy(void){
/*
	GPIO_InitTypeDef GPIOInitStruct;
	volatile int IsBusy, IsOn, IsInReset;

	//set zeros at data pins
	mylcd_SetDataPins(0);

	//set data status pins as inputs
	GPIOInitStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIOInitStruct.GPIO_OType = GPIO_OType_PP;
	GPIOInitStruct.GPIO_Pin = MYLCD_STATUSPINS;
	GPIOInitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIOInitStruct.GPIO_Speed = GPIO_Speed_Level_3;
	GPIO_Init(MYLCD_PORT, &GPIOInitStruct);

	//select both chips
	mylcd_SelectChip(MYLCD_SELECTCHIP_BOTH);

	//select command
	mylcd_SelectCmd(MYLCD_SELECTCMD_INSTRUCTION, MYLCD_SELECTCMD_READ);

	//confirm
	mylcd_Strobe();

	//read status bit
	IsBusy = GPIO_ReadInputDataBit(MYLCD_PORT, MYLCD_PIN_D7);
	IsOn = GPIO_ReadInputDataBit(MYLCD_PORT, MYLCD_PIN_D5);
	IsInReset = GPIO_ReadInputDataBit(MYLCD_PORT, MYLCD_PIN_D4);

	//set data pins as output again
	GPIOInitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIOInitStruct.GPIO_OType = GPIO_OType_PP;
	GPIOInitStruct.GPIO_Pin = MYLCD_STATUSPINS;
	GPIOInitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIOInitStruct.GPIO_Speed = GPIO_Speed_Level_3;
	GPIO_Init(MYLCD_PORT, &GPIOInitStruct);

	return IsBusy;
*/
	mylcd_wait();
	return 0;
}


#define MYLCD_XMASK		0x3F
//x = <0;63>
void lcd_SetX(unsigned x){
	x &= MYLCD_XMASK;
	x |= LCD_PIN_D6;
	lcd_SelectChip(LCD_SELECTCHIP_BOTH);
	lcd_SelectCmd(LCD_SELECTCMD_INSTRUCTION, LCD_SELECTCMD_WRITE);
	lcd_SetDataPins(x);
	while(lcd_IsControllerBusy());
	lcd_Strobe();
}

#define MYLCD_PAGEMASK	0x7
//y = <0;7>
void lcd_SetPage(unsigned page){
	page &= MYLCD_PAGEMASK;
	page |= (LCD_PIN_D7 | LCD_PIN_D5 | LCD_PIN_D4 | LCD_PIN_D3);
	lcd_SelectChip(LCD_SELECTCHIP_BOTH);
	lcd_SelectCmd(LCD_SELECTCMD_INSTRUCTION, LCD_SELECTCMD_WRITE);
	lcd_SetDataPins(page);
	while(lcd_IsControllerBusy());
	lcd_Strobe();
}


#define MYLCD_DATAMASK	0xFF
void lcd_WriteData(unsigned data){
	lcd_SelectCmd(LCD_SELECTCMD_DATA, LCD_SELECTCMD_WRITE);
	lcd_SetDataPins(data);
	lcd_Strobe();
}






