// Header:
// File Name:					mygpio.c
// File Description:
// Author:						Stanislav Subrt
// Date:							2013
// Todo:
// Notes:


#include "potmet.h"
#include "main.h"



//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTION DECLARATIONS ////////////////////////////
//////////////////////////////////////////////////////////////////////

static int pm_MethodStart(pm_MeasInfoTd *p);
static int pm_MethodStop(void);

static void pm_MeasAdvance(void);
static void pm_StartCountingMs(void);
static void pm_StopCountingMs(void);
static int pm_GetMethodDuration(pm_MeasInfoTd *p);


static void pm_SaveLatestMeasurement(pm_MeasCycleTd* cycleInfoStruct, double latestPotmetVoltage);
static void pm_AdvanceMeasurementCycle(pm_MeasCycleTd* cycleInfoStruct);
static void pm_ClearReadyIndexesFromCyclesStorage(pm_MeasCycleTd* cycleInfoStruct);


//complete cycle callbacks
static void EndOfTestCallback(void);
static void EndOfCrackDetectionCallback(void);
static void EndOfCrackMeasurementCallback(void);
static void EndOfRefMeasCallback(void);

//single measurement phase callbacks
static void CommonEndOfPhaseCallback(void);
static void TestPhaseEndCallback(void);
static void RefPhaseEndCallback(void);
static void CrackDetPhaseEndCallback(void);
static void CrackMeasPhaseEndCallback(void);
static double mypotmet_ConvertADCVoltageToPotmetVoltage(double ADCVoltage);


static void FillSingleMeasurementStructFromResultlist(pm_SingleMeasDataTd *dest);
static void SendSingleMeasData(pm_SingleMeasDataTd* measData);

/////////////////////////////////////////////////////////////////////
////////// GLOBALS AND STATICS////////////////////////////////////////
//////////////////////////////////////////////////////////////////////



pm_CycleResultTd CycleResultStruct;		//define structures holding measurement cycle info - number of measurements in cycle, structure describing measurement and its cycles and structure to store values

static short AdcBuffer[VALBUFFERSIZE];			//define buffer to store adc values from each single measurement

static pm_MeasCycleTd* pRunningCycle = 0;
static pm_MeasCycleTd* pLatestCycle = 0;



pm_SingleMeasDataTd pm_CrackReferenceMeasurementData;	//struct to store reference values
pm_SingleMeasDataTd pm_CrackDetectionData;			//struct to store actual values
pm_SingleMeasDataTd pm_CrackMeasData;				//struct to store actual values for crack length measurement


#define GLOBALDELAY		1000

//Potmet setup
#define SAMPS						500	//number of samples taken
#define SAMPTIME					1		//must be higher or equal to one ms
//samps*samptime should equal PERIOD (sampletime must be always N * PERIOD... n=1,2,3,4...

//Syndet setup
#define PERIOD						0




//////////////////////////////////////////////////////////////////////
////////// MEASUREMENT DEFINITIONS ///////////////////////////////////
//////////////////////////////////////////////////////////////////////

#define ENDOFMEASUREMENT	0, 0, 0, 0, 0, 0, 0

//////////////////////////////////////////////////////////////////////
////////// DC TEST ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


 pm_PhaseInfoTd	DCTestPhaseList[] = {		//beware if there is no pointer to data buffer, whole measurement will be terminated
//	pol			chan	delay		ptr to buf		samps		samps_delay						ptrCallback
	 {0,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						TestPhaseEndCallback},
	 {1,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						TestPhaseEndCallback},

	 {0,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						TestPhaseEndCallback},
	 {1,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						TestPhaseEndCallback},

	 {ENDOFMEASUREMENT}
 };

//define Method structure - list with phases, syndet period (zero means that syndet is off) and three internal values that has to be set to zero
pm_MeasInfoTd	DCTestMethod = { DCTestPhaseList, PERIOD, 			0,	0,		0 };

//define measurement cycle struct - number of measurements in one cycle, method of measurement, result structure, pointer to cycle ready callback, mandatory ZERO (Waiting for callback handling) and mandatory ZERO(is cycle ready).
pm_MeasCycleTd  pm_DCTestMeasCycle = {1, &DCTestMethod, &CycleResultStruct, EndOfTestCallback, 0, 0};



//////////////////////////////////////////////////////////////////////
////////// DC REFERENCE MEASUREMENT //////////////////////////////////
//////////////////////////////////////////////////////////////////////

 pm_PhaseInfoTd	DCRefMeasPhaseList[] = {		//beware if there is no pointer to data buffer, whole measurement will be terminated
//	pol			chan	delay		ptr to buf		samps		samps_delay						ptrCallback
	 {0,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						RefPhaseEndCallback},
	 {1,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						RefPhaseEndCallback},

	 {0,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						RefPhaseEndCallback},
	 {1,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						RefPhaseEndCallback},

	 {ENDOFMEASUREMENT}
 };

 //define Method structure - list with phases, syndet period (zero means that syndet is off) and three internal values that has to be set to zero
pm_MeasInfoTd	DCRefMeasMethod = { DCRefMeasPhaseList, PERIOD, 			0,	0,		0 };

//define measurement cycle struct - number of measurements in one cycle, method of measurement, result structure, pointer to cycle ready callback, mandatory ZERO (Waiting for callback handling) and mandatory ZERO(is cycle ready).
pm_MeasCycleTd  pm_DCRefMeasCycle = {1, &DCRefMeasMethod, &CycleResultStruct, EndOfRefMeasCallback, 0, 0};



//////////////////////////////////////////////////////////////////////
////////// DC CRACK DETECTOR /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

 pm_PhaseInfoTd	DCCrackDetectPhaseList[] = {		//beware if there is no pointer to data buffer, whole measurement will be terminated
//	pol			chan	delay		ptr to buf		samps		samps_delay						ptrCallback
	 {0,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackDetPhaseEndCallback},
	 {1,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackDetPhaseEndCallback},

	 {0,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackDetPhaseEndCallback},
	 {1,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackDetPhaseEndCallback},

	 {ENDOFMEASUREMENT}
 };


//define Method structure - list with phases, syndet period (zero means that syndet is off) and three internal values that has to be set to zero
pm_MeasInfoTd	DCCrackDetectMethod = { DCCrackDetectPhaseList, PERIOD, 			0,	0,		0 };

//define measurement cycle struct - number of measurements in one cycle, method of measurement, result structure, pointer to cycle ready callback, mandatory ZERO (Waiting for callback handling) and mandatory ZERO(is cycle ready).
pm_MeasCycleTd  pm_DCCrackDetectMeasCycle = {1, &DCCrackDetectMethod, &CycleResultStruct, EndOfCrackDetectionCallback, 0, 0};



//////////////////////////////////////////////////////////////////////
////////// DC CRACK MEASUREMENT //////////////////////////////////////
//////////////////////////////////////////////////////////////////////

 pm_PhaseInfoTd	DCCrackMeasPhaseList[] = {		//beware if there is no pointer to data buffer, whole measurement will be terminated
//	pol			chan	delay		ptr to buf		samps		samps_delay						ptrCallback
	 {0,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackMeasPhaseEndCallback},
	 {1,		1,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackMeasPhaseEndCallback},

	 {0,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackMeasPhaseEndCallback},
	 {1,		2,		GLOBALDELAY,		AdcBuffer,		SAMPS,		SAMPTIME,						CrackMeasPhaseEndCallback},

	 {ENDOFMEASUREMENT}
 };

//define DCMethod structure - list with phases, syndet period (zero means that syndet is off) and three internal values that has to be set to zero
pm_MeasInfoTd	DCCrackLengthMethod = { DCCrackMeasPhaseList, PERIOD, 			0,	0,		0 };

//define measurement cycle struct - number of measurements in one cycle, method of measurement, result structure, pointer to cycle ready callback, mandatory ZERO (Waiting for callback handling) and mandatory ZERO(is cycle ready).
pm_MeasCycleTd  pm_DCCrackLengthMeasCycle = {1, &DCCrackLengthMethod, &CycleResultStruct, EndOfCrackMeasurementCallback, 0, 0};



//////////////////////////////////////////////////////////////////////
////////// MEASUREMENT METHOD FUNCTIONS //////////////////////////////
//////////////////////////////////////////////////////////////////////

//fires measurement specified in parameter
static volatile pm_MeasInfoTd* pRunningMethod = 0;
static int pm_MethodStart(pm_MeasInfoTd *p){
	if(pRunningMethod) return -1;			//return error flag already running

	pRunningMethod = p;						//set flag about running measurement
	pRunningMethod->IsStopRequested = 0;	//clear stop request

	pm_StartCountingMs();				//setup timer to periodically interrupt in 1ms
	pm_MeasAdvance();					//first MeasAdvance is just config - do it now. First interrupt will be measurement after specified samps_delay
	return 0;								//ok
}



int pm_MethodStop(void){
	if(pRunningMethod == 0) return -1;
	pRunningMethod->IsStopRequested = 1;	//set stop request flag (is cleared always during MethodStart)
	return 0;
}



#define MYPOTMET_METHOD_NOTRUNNING	-1
#define MYPOTMET_METHOD_INVALIDHANDLE	-2
static int pm_GetMethodDuration(pm_MeasInfoTd *p){
	int i = 0;
	int TimeTotalMs = 0;
	if( p == 0 ) return MYPOTMET_METHOD_INVALIDHANDLE;


	while( p->PhaseList[i].pAdcBuffer != 0 ){											//run through all phases
		TimeTotalMs += p->PhaseList[i].Delay;											//add time for settling
		TimeTotalMs += p->PhaseList[i].DelayBetweenSamples * p->PhaseList[i].Samples;	//add time for sampling
		i++;
	}
	return TimeTotalMs;
}



//////////////////////////////////////////////////////////////////////
////////// MEASUREMENT CYCLE FUNCTIONS ///////////////////////////////
//////////////////////////////////////////////////////////////////////

//starts measurement cycle. Checks if cycle is already running and calls methodstart which does the measurement. In measurement callback advancemeasurementcycle is called
//called by user interaction (button, radio)
int pm_StartMeasCycle(pm_MeasCycleTd* cycleInfoStruct){

	if(pRunningCycle != 0) {
		return POTMET_CYCLE_ALREADY_RUNNING;
	}

	pRunningCycle = cycleInfoStruct;
	pLatestCycle = pRunningCycle;
	cycleInfoStruct->ResultsStruct->Index = 0;	//reset storage array index to zero and advance in measurement
	cycleInfoStruct->IsReady = 0;				//reset ready flag
	pm_ClearReadyIndexesFromCyclesStorage(cycleInfoStruct);	//clear ready flags from storage

	pm_MethodStart(cycleInfoStruct->InfoStruct);
	return POTMET_CYCLE_OK;
}



int pm_StopMeasCycle(void){
	if( pRunningCycle == 0 ) return POTMET_CYCLE_NOT_RUNNING;

	pm_MethodStop();
	pm_SelectChannel(-1);			//disable all channels to minimize current consumption
	pRunningCycle = 0;
	return POTMET_CYCLE_OK;
}



//Called from main to invoke callback after measurement phase. Measurement is waiting after each phase for this
void pm_InvokeCallbacks(void){
	void (*f)();				//pointer to callback

	if(pRunningMethod != 0){						//measurement is running so this is end of phase callback
		if( pRunningMethod->WaitingForCallbackHandling == 1){								//if we are waiting for callback
			f = pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].PhaseCallback;	//load callback from measurement structure
			if(f != 0) {																	//if callback function pointer is valid
				f();																									//invoke callback
			}
			pRunningMethod->WaitingForCallbackHandling = -1;								//ack callback is ready
		}
	}

	if(pLatestCycle != 0){				//cycle is running or is finished and data should be ready
		if(pLatestCycle->WaitingForCallbackHandling && pLatestCycle->IsReady){		//check if cycle is waiting for callback and just to be sure if data is ready as well
			f = pLatestCycle->CycleReadyCallback;
			if(f !=0){
				f();
			}
			pLatestCycle->WaitingForCallbackHandling = 0;
		}
	}
}



void pm_SendLatestCycleResults(pm_MeasCycleTd* cycleInfoStruct){
	int i, j, MeasurementsInCycle;
	int Delay;

	double Value;

	char tmp[512], val[64];

	if(cycleInfoStruct == 0){
		return;
	}
	else if(cycleInfoStruct->IsReady == 0){
		return;
	}


	MeasurementsInCycle = cycleInfoStruct->MeasurementsInCycle;
	for( i=0; i<MeasurementsInCycle ; i++){

		if(cycleInfoStruct->ResultsStruct->ResultsList[i].ReadyIndex){	//if current measurement is ready
			sprintf(tmp, "<");
			for( j=0; j<PHASES_IN_MEASUREMENT ; j++ ){
				Value = cycleInfoStruct->ResultsStruct->ResultsList[i].MeasResultsArray[j];
				sprintf(val, "%06.3lf ", Value);
				strcat(tmp, val);
			}
			sprintf(val, ">\r\n");
			strcat(tmp,val);
			comm_SendData(tmp, strlen(tmp));
			for( Delay=0; Delay<20000; Delay++);			//10000 not enough for 40min measurement
		}
	}
}



void pm_SendLatestResultsTest(void){
	int i, Delay;
	char tmp[] = "<982.932 -269.081 1275.958 -217.146 980.131 -435.684 1086.617 -534.945\r\n>";

	for( i=0; i<100 ; i++){


			comm_SendData(tmp, strlen(tmp));
			for( Delay=0; Delay<20000; Delay++);
	}
}



//returns time duration of current cycle
int pm_GetRunningCycleDuration(void){
	if( pRunningCycle == 0) return POTMET_CYCLE_NOT_RUNNING;

	//return product of number of measurement in cycle and each of cycles duration in ms
	return ( pRunningCycle->MeasurementsInCycle * pm_GetMethodDuration(pRunningCycle->InfoStruct) );
}



//saves data from latest measurement phase
//called from phase callback
static void pm_SaveLatestMeasurement(pm_MeasCycleTd* cycleInfoStruct, double latestPotmetVoltage){
	cycleInfoStruct->ResultsStruct->ResultsList[cycleInfoStruct->ResultsStruct->Index].MeasResultsArray[pRunningMethod->ActivePhaseIndex] = latestPotmetVoltage;
	cycleInfoStruct->ResultsStruct->ResultsList[cycleInfoStruct->ResultsStruct->Index].ReadyIndex++;
}



//advances in measurement cycle - increments measurement index and starts method
//called from timer ISR after end of latest phase in single measurement
static void pm_AdvanceMeasurementCycle(pm_MeasCycleTd* cycleInfoStruct){

	if(cycleInfoStruct == 0) return;

	cycleInfoStruct->ResultsStruct->Index++;			//increment index of list with structs containing measurement data for each channel


	if( (cycleInfoStruct->ResultsStruct->Index) < (cycleInfoStruct->MeasurementsInCycle)) {		//if there are still measurements to do
		pm_MethodStart(cycleInfoStruct->InfoStruct);										//start new measurement
	}
	else {
		pRunningCycle = 0;
		cycleInfoStruct->IsReady = 1;		//set data ready flag
		cycleInfoStruct->WaitingForCallbackHandling = 1;
		pm_SelectChannel(-1);
	}
}



static void pm_ClearReadyIndexesFromCyclesStorage(pm_MeasCycleTd* cycleInfoStruct){
	int i;
	for ( i=0; i<(cycleInfoStruct->MeasurementsInCycle); i++ ){				//in each of measurements structs stored in list
		cycleInfoStruct->ResultsStruct->ResultsList[i].ReadyIndex = 0;		//clear number of ready/measured channels
	}
}



pm_MeasResultsTd* pm_GetSelectedMeasurementResults(pm_MeasCycleTd* cycleInfoStruct, int measurementIndex){

	return &(cycleInfoStruct->ResultsStruct->ResultsList[measurementIndex]);
}



pm_MeasResultsTd* pm_GetCurrentMeasurementResults(pm_MeasCycleTd* cycleInfoStruct){

	if( cycleInfoStruct == 0 ) return 0;

	if(pRunningCycle == 0){
		return &(cycleInfoStruct->ResultsStruct->ResultsList[cycleInfoStruct->ResultsStruct->Index - 1]);
	}
	else{
		return &(cycleInfoStruct->ResultsStruct->ResultsList[cycleInfoStruct->ResultsStruct->Index]);
	}

}



#define MYPOTMET_CYCLE_SPECERROR	-10
int pm_GetRunningCycleProgress(void){
	int Total, Ready;

	if( pRunningCycle == 0 ) return POTMET_CYCLE_NOT_RUNNING;

	if( pRunningCycle->IsReady ) return 100;		//cycle is ready return 100%

	Total = pRunningCycle->MeasurementsInCycle;
	Ready = pRunningCycle->ResultsStruct->Index;
	if( Total <= 0 ) return MYPOTMET_CYCLE_SPECERROR;

	return (int)( ( (float)Ready / (float)Total ) * 100 );
}



pm_MeasCycleTd* pm_GetRunningCycle(void){
	return pRunningCycle;
}



pm_MeasCycleTd* pm_GetLatestCycle(void){
	return pLatestCycle;
}



void mypotmet_ClearLatestCycle(void){
	pLatestCycle = 0;
}



void mypotmet_ClearLatestStorageData(void){
	pm_MeasCycleTd *measCycle = pm_GetLatestCycle();
	if( measCycle == 0 ){
		return;
	}
	pm_ClearReadyIndexesFromCyclesStorage(measCycle);
}



char* pm_GetApertureStatus(void){
	static char tmp[128];
	int CycleDuration;
	int MeasCycleProgress = pm_GetRunningCycleProgress();

	if(MeasCycleProgress == POTMET_CYCLE_NOT_RUNNING){
		if(pm_GetLatestCycle() == 0){		//if measurement is not running and there is no pointer to previous measurement, this means that there is also no data waiting and aperture is in status ready
			return MAIN_APERTUREREADY;
		}
		else{
			if(pm_GetLatestCycle()->IsReady){
				return MAIN_MEASCOMPLETE;
			}
			else{
				return MAIN_MEASARBORTED;
			}
		}
	}
	else if(MeasCycleProgress >= 0){	//measurement is running

		CycleDuration = pm_GetRunningCycleDuration() / (1000 * 60);		//milliseconds / (1000*60) = minutes
		sprintf(tmp, "%s %03d%%(%03d/%03dmin)\r\n>", MAIN_MEASRUNNING, MeasCycleProgress, (CycleDuration * MeasCycleProgress) / 100, CycleDuration );	//format actual progress into string
		return tmp;
	}
	else{
		return "Error\r\n";
	}
}



//clears latest data storage if measurement is not running
void pm_ClearLatestCycleData(void){

	if(pm_GetRunningCycleProgress() == POTMET_CYCLE_NOT_RUNNING){
		//order is important... first delete latest storage data and then pointer to latest measurement
		mypotmet_ClearLatestStorageData();
		mypotmet_ClearLatestCycle();
	}
}

//////////////////////////////////////////////////////////////////////
////////// PHASE END CALLBACKS ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void CommonEndOfPhaseCallback(void){
	//calculate value and save it into storage structure
	int i;
	double VoltageAdc, VoltagePotmet, avg=0;

	//make average from stored values
	for	(i = 0; i < pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].Samples; i++){
		avg += pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].pAdcBuffer[i];
	}
	avg = avg / (i+1);

	//calculate adc voltage from adc value average and convert it to potmet voltage
	VoltageAdc = adc_ConvertToVoltage(avg);
	VoltagePotmet = mypotmet_ConvertADCVoltageToPotmetVoltage(VoltageAdc);
	pm_SaveLatestMeasurement(pRunningCycle, VoltagePotmet);
}



static void TestPhaseEndCallback(void){
	CommonEndOfPhaseCallback();

	gui_RequestRefresh();
}



static void RefPhaseEndCallback(void){
	CommonEndOfPhaseCallback();

	FillSingleMeasurementStructFromResultlist(&pm_CrackReferenceMeasurementData);

	gui_RequestRefresh();
}



static void CrackDetPhaseEndCallback(void){
	CommonEndOfPhaseCallback();

	FillSingleMeasurementStructFromResultlist(&pm_CrackDetectionData);

	gui_RequestRefresh();
}


static void CrackMeasPhaseEndCallback(void){
	CommonEndOfPhaseCallback();

	FillSingleMeasurementStructFromResultlist(&pm_CrackMeasData);

	gui_RequestRefresh();
}



//////////////////////////////////////////////////////////////////////
////////// CYCLE END CALLBACKS ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////

static void EndOfTestCallback(void){
	gui_RequestRefresh();								//refresh GUI
}



static void EndOfRefMeasCallback(void){
	gui_RequestRefresh();								//refresh GUI
	SendSingleMeasData(&pm_CrackReferenceMeasurementData);
}



static void EndOfCrackDetectionCallback(void){
	gui_RequestRefresh();								//refresh GUI
	SendSingleMeasData(&pm_CrackDetectionData);
}



static void EndOfCrackMeasurementCallback(void){
	gui_RequestRefresh();								//refresh GUI
	SendSingleMeasData(&pm_CrackMeasData);
}



//////////////////////////////////////////////////////////////////////
////////// UTILITIES /////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

static void SendSingleMeasData(pm_SingleMeasDataTd* measData){
	char tmp[256];
	float Ch1P, Ch1M, Ch2P, Ch2M;

	if( measData == 0) return;

	Ch1P = measData->Ch1.PolP.Value;
	Ch1M = measData->Ch1.PolM.Value;
	Ch2P = measData->Ch2.PolP.Value;
	Ch2M = measData->Ch2.PolM.Value;

	sprintf(tmp, "<GD 1>\r\n< %.3f\t%.3f\t%.3f\t%.3f >\r\n", Ch1P, Ch1M, Ch2P, Ch2M);
	comm_SendData(tmp, strlen(tmp));
}



static void FillSingleMeasurementStructFromResultlist(pm_SingleMeasDataTd *dest){
	pm_MeasResultsTd* pResult;
	float Ch1P, Ch1M, Ch2P, Ch2M, Sub1, Sub2;

	//should never happen but just to be sure not to jump into hardfault
	if(pRunningCycle == 0) return;

	//get the pointer to actual result list
	pResult = &(pRunningCycle->ResultsStruct->ResultsList[pRunningCycle->ResultsStruct->Index]);

	//read test results for both channels and polarities and substract thermoelectrical voltage
	Ch2M = pResult->MeasResultsArray[3];
	Ch2P = pResult->MeasResultsArray[2];
	Ch1M = pResult->MeasResultsArray[1];
	Ch1P = pResult->MeasResultsArray[0];
	Sub1 = ( Ch1P - Ch1M ) / 2;
	Sub2 = ( Ch2P - Ch2M ) / 2;

	//according to ready index fill dest structure
	switch(pResult->ReadyIndex){
		case 4:
			dest->Ch2.PolM.Value = Ch2M;
			dest->Ch2.PolM.IsValid = 1;

			dest->Ch2.Sub.Value = Sub2;
			dest->Ch2.Sub.IsValid = 1;

		case 3:
			dest->Ch2.PolP.Value = Ch2P;
			dest->Ch2.PolP.IsValid = 1;
		case 2:
			dest->Ch1.PolM.Value = Ch1M;
			dest->Ch1.PolM.IsValid = 1;
			dest->Ch1.Sub.Value = Sub1;
			dest->Ch1.Sub.IsValid = 1;
		case 1:
			dest->Ch1.PolP.Value = Ch1P;
			dest->Ch1.PolP.IsValid = 1;
	}
}


void pm_ClearSingleMeasurementStruct(pm_SingleMeasDataTd *dest){
			dest->Ch2.PolM.IsValid = 0;
			dest->Ch2.PolP.IsValid = 0;
			dest->Ch2.Sub.IsValid = 0;

			dest->Ch1.PolM.IsValid = 0;
			dest->Ch1.PolP.IsValid = 0;
			dest->Ch1.Sub.IsValid = 0;
}



//////////////////////////////////////////////////////////////////////
////////// INTERNAL FUNCTIONS ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//ISR of timer3 - timing of measuremet, taking samples etc
static void pm_MeasAdvance(void){
	static int SamplesCounter = 0;					//counts samples in each phase, zero means phase config time - beware when using this value
	static int TimeToWakeUp = 0;					//counts time delays in ms

	if(pRunningMethod){								//if measurement is running

		if(	pRunningMethod->IsStopRequested	){		//if stop has been requested
			pm_StopCountingMs();				//since this fn is not reentrant there is a risk of entering from timer interrupt while processing from normal thread - therefore we have to disable counting before calling MeasAdvance with stopflag

			//revert all changes to method struct
			pRunningMethod->ActivePhaseIndex = 0;
			pRunningMethod->WaitingForCallbackHandling = 0;
			pRunningMethod->IsStopRequested = 0;
			SamplesCounter = 0;
			TimeToWakeUp = 0;
			pRunningMethod = 0;						//must be last thing done after measurement

			//mygui_RequestLCDRefresh();
			return;
		}

		if(TimeToWakeUp == 0) {						//if we are not waiting
			if(SamplesCounter == 0) {				//this means new phase so we need to set phase params
				pm_SelectChannel(pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].Channel);
				sd_SetPolarity(pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].Polarity);
				sd_SetPeriod(pRunningMethod->Period);


				SamplesCounter++;					//increment sample counter so we know setup is complete
				TimeToWakeUp = pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].Delay;				//set up new delay
			}
			else if (SamplesCounter > pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].Samples){		//this means next phase or end of measurement is happening

				if(pRunningMethod->WaitingForCallbackHandling == 0){		//if we are not waiting for callback but next phase or end of measurement is requested we set flag and start waiting for its processing
					pRunningMethod->WaitingForCallbackHandling = 1;			//set flag towait until callback is handled ( == -1)
				}
				else if(pRunningMethod->WaitingForCallbackHandling==1){
					return; 												//waiting for callback invoking see mypotmet_InvokePhaseDataCallback();
				}
				else{														//callback has been processed
					//mygui_RequestLCDRefresh();								//refresh GUI
					pRunningMethod->ActivePhaseIndex++;						//increment phase index pointing to next phase info
					pRunningMethod->WaitingForCallbackHandling = 0;
					SamplesCounter = 0;										//clear sample counter - setup phase is first to happen
					TimeToWakeUp = 0;

					if( (pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].pAdcBuffer) == 0) {	//if measurement is finished
						pm_StopCountingMs();
						pRunningMethod->ActivePhaseIndex = 0;
	/*----------------------*/
						pRunningMethod = 0;										//must be last thing done after measurement - no method is running
						pm_AdvanceMeasurementCycle(pRunningCycle);		//sngle measurement finished - time to advance in measurement cycle
	/*----------------------*/
					}
				}
			}
			else {															//sample time
				if(SamplesCounter <= VALBUFFERSIZE){						//chech if sample buffer is not full, else just skip saving
					pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].pAdcBuffer[SamplesCounter-1] =
					adc_GetValue();										//read adc value
				}
				SamplesCounter++;											//increment samples counter
				TimeToWakeUp = pRunningMethod->PhaseList[pRunningMethod->ActivePhaseIndex].DelayBetweenSamples;	//asynchronously wait for specified delay between samples
			}
		}
		if (TimeToWakeUp > 0) {												//if time to wake up is positive just lower counter
			TimeToWakeUp--;													//decrement always when measurement is running and timeout has been set
		}
	}
}



//////////////////////////////////////////////////////////////////////
////////// HARDWARE ABSTRACTION //////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//initializes GPIOs for channel selection and timer 2 for 1ms counting

void pm_Init(void) {
	GPIO_InitTypeDef			GPIO_InitStructure;
	NVIC_InitTypeDef 			NVIC_InitStructure;
	TIM_TimeBaseInitTypeDef 	TIM_TimeBaseInitStruct;


	RCC_AHBPeriphClockCmd(POTMET_CHANNELPORT_PERIPHCLOCK, ENABLE);

	GPIO_InitStructure.GPIO_Pin  = POTMET_CHANNELPINS;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(POTMET_CHANNEL_PORT, &GPIO_InitStructure);


	//tim3 for ms counting
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);  			//TIM3 clock enable

	TIM_TimeBaseInitStruct.TIM_Prescaler = 9;						//coreclk/(pres+1) = timclk
	TIM_TimeBaseInitStruct.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInitStruct.TIM_Period = 7200;
	TIM_TimeBaseInitStruct.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseInitStruct.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseInitStruct);

	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
	TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);

	pm_SelectChannel(-1);			//disable all channels to minimize current consumption
}


//selects measurement channel
//PC0 LSB ch, PC1 MSB ch.
//ch = 1,2,3,4, 0 keeps same channel, other values disable all channels
void pm_SelectChannel(int ch){

	if (ch == 0){												//if channel is 0, this means dont change channel
		return;
	}

	GPIO_ResetBits(POTMET_CHANNEL_PORT, POTMET_CHANNELPINS);			//now switch off all channels

	if(ch==1){													//select channel
		GPIO_SetBits(POTMET_CHANNEL_PORT, POTMET_CHANNEL1);
	}
	else if(ch==2){
		GPIO_SetBits(POTMET_CHANNEL_PORT, POTMET_CHANNEL2);
	}
	else if(ch==3){
		GPIO_SetBits(POTMET_CHANNEL_PORT, POTMET_CHANNEL3);
	}
	else if(ch==4){
		GPIO_SetBits(POTMET_CHANNEL_PORT, POTMET_CHANNEL4);
	}
}



//recalculates measured ADC voltage to real voltage at divider
static double mypotmet_ConvertADCVoltageToPotmetVoltage(double ADCVoltage){
	return ( (double) ADCVoltage * ( ADC_R1*ADC_R2 + ADC_R2*ADC_R3 + ADC_R1*ADC_R3 ) - ( ADC_R2 * ADC_R3 * ADC_VREF ) ) / ( ADC_R1 * ADC_R3 );	//formula written also in vypocet delicky pred adc.xlsx
}


void TIM3_IRQHandler(void){
	if(TIM_GetITStatus(TIM3, TIM_IT_Update) == SET){		//this happens every each 1ms
		pm_MeasAdvance();
		TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
	}
}

static void pm_StartCountingMs(void){
		TIM_Cmd(TIM3, ENABLE);
}

static void pm_StopCountingMs(void){
		TIM_Cmd(TIM3, DISABLE);
		TIM_SetCounter(TIM3, 0);
}
